<?php
/*
 * PunchCommerce Shopware-Plugin
 *
 * @copyright: Copyright (c) netzdirektion | Gesellschaft für digitale Wertarbeit mbH, 2021
 *
 * @link: https://www.punchcommerce.de
 * @link: https://netzdirektion.de
 */

namespace PunchCommerce\Components;

use JTL\Campaign;
use JTL\Cart\Cart;
use JTL\Cart\CartHelper;
use JTL\Customer\Customer;
use JTL\DB\DbInterface;
use JTL\DB\ReturnType;
use JTL\Session\Frontend;
use JTL\Shop;
use PunchCommerce\Components\Exceptions\InvalidRefererException;
use PunchCommerce\Components\Exceptions\InvalidTokenException;
use PunchCommerce\Components\Exceptions\InvalidUidException;
use PunchCommerce\Components\Exceptions\MissingUidException;
use PunchCommerce\Components\Requests\AuthenticationRequest;

/**
 * Class AuthenticationService
 * @package NetzdPunchCommerce\Components
 */
class AuthenticationService
{

    private Frontend $session;
    private DbInterface $db;
    private string $key = 'cKundenNr';

    public function __construct()
    {
        $this->session = Frontend::getInstance();
        $this->db = Shop::Container()->getDB();
    }

    public function authenticateShopUser(AuthenticationRequest $request, string $key = null): bool
    {
        if (!empty($key)) {
            $this->key = $key;
        }

        $shopUser = $this->getShopUserByUid($request->getUid());

        if ($shopUser === null) {
            throw new InvalidUidException('Customer was not found or credentials were invalid');
        }

        $this->loginUser($shopUser);

        $_SESSION['Warenkorb'] = new Cart();

        $this->session::set('punchcommerce_session_id', $request->getSId());
        $this->session::set('punchcommerce_session_type', $request->getType());
        $this->session::set('punchcommerce_session', true);

        return true;
    }

    private function getShopUserByUid(string $uid = null): ?Customer
    {
        if ($uid === null) {
            throw new MissingUidException('Parameter uID is missing');
        }

        $sql = "SELECT DISTINCT kKunde FROM tkunde WHERE {$this->key} = :uId";
        $customers = $this->db->queryPrepared($sql, ['uId' => $uid], ReturnType::ARRAY_OF_OBJECTS);

        if (count($customers) > 0) {
            $customer = reset($customers);
            $customer = new Customer($customer->kKunde);
            return $customer;
        }

        return null;
    }

    private function loginUser(Customer $customer)
    {
        unset($_SESSION['showLoginCaptcha']);
        session_regenerate_id();
        $this->updateSession();
        $this->session = $this->session::getInstance();
        $this->session->setCustomer($customer);
        Shop::Container()->getLinkService()->reset();
    }

    private function updateSession(): void
    {
        unset(
            $_SESSION['Zahlungsart'],
            $_SESSION['Versandart'],
            $_SESSION['Lieferadresse'],
            $_SESSION['ks'],
            $_SESSION['VersandKupon'],
            $_SESSION['NeukundenKupon'],
            $_SESSION['Kupon'],
            $_SESSION['oKategorie_arr_new']
        );
    }

    public function isPunchCommerceSession()
    {
        $this->init();
        return $this->session->offsetExists('punchcommerce_session');
    }

    private function isAllowedReferer(string $referer)
    {
        $allowed = [
            'https://punchcommerce.local',
            'https://punchcommerce.local/',
            'https://www.punchcommerce.de',
            'https://www.punchcommerce.de/',
            'https://punchcommerce.de',
            'https://punchcommerce.de/'
        ];

        if (!array_key_exists($referer, array_flip($allowed))) {
            return false;
        }

        return true;
    }
}
